<?php
require_once 'dto/Card.php';
require_once(WebOrb . "Util/ORBDateTime.php");

class CardDao {

    private $pdo;

    /**
     * コンストラクタ。
     */
    function __construct($dsn) {
        $this->pdo = new PDO($dsn);
    }

    function createTableIfNotExists() {
        $sql = "CREATE TABLE IF NOT EXISTS card (
            id INTEGER,
            last_name TEXT,
            first_name TEXT,
            last_name_yomi TEXT,
            first_name_yomi TEXT,
            company TEXT,
            tel TEXT,
            email TEXT,
            zip TEXT,
            address TEXT,
            url TEXT,
            birthday INTEGER,
            portrait_url TEXT,
            PRIMARY KEY (id)
        )";
        $this->pdo->exec($sql);
    }

    private function createCardObject($row) {
        $card = new Card();
        $card->id = $row['id'];
        $card->email = $row['email'];
        $card->lastName = $row['last_name'];
        $card->firstName = $row['first_name'];
        $card->lastNameYomi = $row['last_name_yomi'];
        $card->firstNameYomi = $row['first_name_yomi'];
        $card->company = $row['company'];
        $card->tel = $row['tel'];
        $card->zip = $row['zip'];
        $card->address = $row['address'];
        $card->url = $row['url'];
        $card->portraitUrl = $row['portrait_url'];
        if ($row['birthday'] > 0) {
            $card->birthday = new ORBDateTime($row['birthday'], 0);
        }
        return $card;
    }

    function findById($id) {
        $sql = "SELECT * FROM card WHERE id=?";
        $stmt = $this->pdo->prepare($sql);
        $stmt->execute(array($id));
        if ($row = $stmt->fetch(PDO::FETCH_ASSOC)) {
            return $this->createCardObject($row);
        }
        return null;
    }

    function findAll() {
        $ret = array();
        $sql = "SELECT * FROM card ORDER BY last_name_yomi, first_name_yomi";
        $stmt = $this->pdo->prepare($sql);
        $stmt->execute();
        while ($row = $stmt->fetch(PDO::FETCH_ASSOC)) {
            $card = $this->createCardObject($row);
            array_push($ret, $card);
        }
        return $ret;
    }

    function insert(Card $card) {
        $sql = "INSERT INTO card(
                last_name, first_name, last_name_yomi, first_name_yomi,
                company, tel, email, zip, address, url, birthday, portrait_url)
                VALUES(?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
        $stmt = $this->pdo->prepare($sql);
        $stmt->execute(array(
            $card->lastName,
            $card->firstName,
            $card->lastNameYomi,
            $card->firstNameYomi,
            $card->company,
            $card->tel,
            $card->email,
            $card->zip,
            $card->address,
            $card->url,
            $card->getBirthdayTimestamp(),
            $card->portraitUrl
        ));
        return $this->pdo->lastInsertId();
    }

    function update(Card $card) {
        $sql = "UPDATE card SET
                last_name=?, first_name=?, last_name_yomi=?, first_name_yomi=?,
                company=?, tel=?, email=?, zip=?, address=?, url=?, birthday=?,
                portrait_url=? WHERE id=?";
        $stmt = $this->pdo->prepare($sql);
        $stmt->execute(array(
            $card->lastName,
            $card->firstName,
            $card->lastNameYomi,
            $card->firstNameYomi,
            $card->company,
            $card->tel,
            $card->email,
            $card->zip,
            $card->address,
            $card->url,
            $card->getBirthdayTimestamp(),
            $card->portraitUrl,
            $card->id
        ));
        return $card->id;
    }

    function delete(Card $card) {
        $sql = "DELETE FROM card WHERE id=?";
        $stmt = $this->pdo->prepare($sql);
        $stmt->execute(array($card->id));
    }

}
?>

