<?php
/*******************************************************************
 * FlexRemotingServiceConfig.php
 * Copyright (C) 2006 Midnight Coders, LLC
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * The software is licensed under the GNU General Public License (GPL)
 * For details, see http://www.gnu.org/licenses/gpl.txt.
 ********************************************************************/
 

 require_once(WebOrb . "Util/Invocation.php");
 require_once(WebOrb . "Util/ObjectFactories.php"); 
 require_once(WebOrb . "Util/XmlUtil.php"); 
 require_once(WebOrb . "Security/AccessConstraint.php");  
 
class FlexRemotingServiceConfig
{
    const REMOTINGSERVICE_FILE = "remoting-config.xml";
		
    public static function configure($orbConfig) 
    {
        $dom = new DomDocument();
        $dom->load($orbConfig->getFlexConfigPath() . FlexRemotingServiceConfig::REMOTINGSERVICE_FILE);
        
		$root = $dom->documentElement;
		$destinationNodes = $root->getElementsByTagName( "destination" );
		
        foreach($destinationNodes as $destElement)
        {

          $destinationId = $destElement->getAttribute( "id" );
          $props = $destElement->getElementsByTagName( "properties" )->item(0);
          $source = XmlUtil::getElementText($props,"source");
          $scope = XmlUtil::getElementText( $props, "scope" );
          
          $context = array();
          
          if($scope != null && strlen(trim($scope)) > 0)
          {
              $context[ORBConstants::ACTIVATION] = $scope;
          }
          
          $orbConfig->getServiceRegistry()->_addMapping($destinationId, $source, $context );
          
          
          $securityNodes = $destElement->getElementsByTagName( "security" );
          
          if($securityNodes != null && $securityNodes->length > 0 )
          {
            
             $securityElement = $securityNodes->item(0);
             $securityConstraintNode = $securityElement->getElementsByTagName( "security-constraint" )->item(0);
             $rolesNode = $securityConstraintNode->getElementsByTagName( "roles" )->item(0);
             $rolesNodeList = $rolesNode->getElementsByTagName( "role" );
             $constraint = new AccessConstraint($source . "_constraint", "grant" );
             
             for($i = 0; $i < $rolesNodeList->length; $i++)             
                $constraint->addRole($rolesNodeList->item($i)->textContent);
            
             $security = $orbConfig->getSecurity();
             $constraintsList = &$security->getConstraintsList();
             $constraintsList[$constraint->getName()] = $constraint;
             
             $constraints = array($constraint->getName());
 
             
             $security->secureResource($source,$constraints,null);
             
          }
        }
    }
}
?>