<?php
/*******************************************************************
 * ORBConfig.php
 * Copyright (C) 2006 Midnight Coders, LLC
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * The software is licensed under the GNU General Public License (GPL)
 * For details, see http://www.gnu.org/licenses/gpl.txt.
 ********************************************************************/



require_once(WebOrb . "Protocols/ProtocolRegistry.php");
require_once(WebOrb . "Protocols/AMFMessageFactory.php");
require_once(WebOrb . "Handlers/ObjectHandler.php");
require_once(WebOrb . "Handlers/Handlers.php");
require_once(WebOrb . "Util/Logging/Log.php");
require_once(WebOrb . "Registry/ServiceRegistry.php");
require_once(WebOrb . "Config/ServicesConfigHandler.php");
require_once(WebOrb . "Config/ClassMappingsHandler.php");
require_once(WebOrb . "Config/FlexRemotingServiceConfig.php");
require_once(WebOrb . "Types/Types.php");
require_once(WebOrb . "Security/ORBSecurity.php");
require_once(WebOrb . "Security/WebORBAuthorizationHandler.php");
require_once(WebOrb . "Security/SecurityConfigHandler.php");
require_once(WebOrb . "Security/AclConfigHandler.php");


class ORBConfig
{

    static private $s_instance = NULL;

    static public function getInstance()
    {
        if (is_null(self::$s_instance))
        {
            self::$s_instance = new ORBConfig();
            self::$s_instance->initialize();
        }
        return self::$s_instance;
    }

    private $m_protocolRegistry;
    private $m_servicePath;
    private $m_serviceRegistry;
    private $m_logLevels;
    private $m_handlers;
    private $m_typeMapper;
    private $m_security;
    private $m_configs;


    private function __construct()
    {
        $this->m_protocolRegistry = new ProtocolRegistry();
        $this->m_handlers = new Handlers();
        $this->m_typeMapper = new Types();
        $this->m_security = new ORBSecurity();
        
        $this->m_configs = array();
    }
    
    public function getSecurity()
    {
      return $this->m_security;
    }
    
    public function getConfig($configPath)
    {
      return $this->m_configs[$configPath];
    }

    public function getProtocolRegistry()
    {
        return $this->m_protocolRegistry;
    }

    public function getServicePath()
    {
        return $this->m_servicePath;
    }

    public function getLogLevels() 
    {
        return $this->m_logLevels;
    }

    public function getHandlers()
    {
        return $this->m_handlers;
    }

    public function getServiceRegistry()
    {
        return $this->m_serviceRegistry;
    }

    public function getTypeMapper()
    {
      return $this->m_typeMapper;
    }

    public function getFlexConfigPath()
    {
      return WebOrb . "WEB-INF" . DIRECTORY_SEPARATOR . "flex" . DIRECTORY_SEPARATOR;
    }
    
    public function initialize()
    {
        Log::log(LoggingConstants::DEBUG, "Called.");

        $dom = new DomDocument();
        $dom->load( WebOrb . "weborb-config.xml");
        $this->m_servicePath = $dom->documentElement->getAttribute('servicePath');

        
        // Initialize handlers
        // TODO: $handlers->addInvocationHandler(new ObjectHandler());

        // Initialize protocols
        $this->m_protocolRegistry->addMessageFactory(new AMFMessageFactory());
    

        $this->m_handlers->addInvocationHandler(new ObjectHandler());
        
        $this->m_serviceRegistry = new ServiceRegistry();

        $servicesConfigHandler = new ServicesConfigHandler();
        $servicesConfigHandler->configure(null, null, $dom->documentElement);

        $classMappingHandler = new ClassMappingsHandler();
        $classMappingHandler->configure(null, null, $dom->documentElement);
        
        $this->m_security->setAuthorizationHandler(new WebORBAuthorizationHandler());
        
        $securityConfigHandler = new SecurityConfigHandler();
        $securityConfigHandler->configure(null,$this,$dom->documentElement);
        
        $aclConfigHandler = new AclConfigHandler();
        $aclConfigHandler->configure(null,$this,$dom->documentElement);
        
        $this->m_configs["weborb/security"] = $securityConfigHandler;
        $this->m_configs["weborb/acl"] = $aclConfigHandler;

        FlexRemotingServiceConfig::configure($this);
        
    }

}

?>
