<?php
/*******************************************************************
 * Dispatchers.php
 * Copyright (C) 2006 Midnight Coders, LLC
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * The software is licensed under the GNU General Public License (GPL)
 * For details, see http://www.gnu.org/licenses/gpl.txt.
 ********************************************************************/



require_once(WebOrb . "Config/ORBConfig.php");
require_once(WebOrb . "Dispatch/Invoker.php");
require_once(WebOrb . "Dispatch/V3Dispatcher.php");
require_once(WebOrb . "Message/Request.php");
require_once(WebOrb . "Util/Logging/Log.php");

final class Dispatchers
{
    static private $s_instance = NULL;
    
    private $m_dispatchers = array();

    private function __construct() 
    {
        $this->m_dispatchers[] = new V3Dispatcher();
        $this->m_dispatchers[] = new Invoker();
    }

    static public function getInstance() 
    {
        if (is_null(self::$s_instance)) 
        {
            self::$s_instance = new Dispatchers();
        }

        return self::$s_instance;
    }

    static public function dispatch(Request $request) 
    {
        Log::log(LoggingConstants::DEBUG, "Called.");

        $instance = self::getInstance();

        for($i = 0; $i < sizeof($instance->m_dispatchers); $i++)
        {
            $dispatcher = $instance->m_dispatchers[$i];

            if($dispatcher->dispatch($request))
            {
              return true;  
            }
        }
        
        return false;
    }
}

?>
