<?php
/*******************************************************************
 * Invoker.php
 * Copyright (C) 2006 Midnight Coders, LLC
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * The software is licensed under the GNU General Public License (GPL)
 * For details, see http://www.gnu.org/licenses/gpl.txt.
 ********************************************************************/



require_once(WebOrb . "Dispatch/IDispatch.php");
require_once(WebOrb . "Config/ORBConfig.php");
require_once(WebOrb . "Handlers/Handlers.php");
require_once(WebOrb . "Handlers/IInvocationHandler.php");
require_once(WebOrb . "Message/Body.php");
require_once(WebOrb . "Message/Request.php");
require_once(WebOrb . "Util/Value.php");
require_once(WebOrb . "Registry/ServiceRegistry.php");
require_once(WebOrb . "Exceptions/ServiceException.php");

class Invoker
    implements IDispatch
{

    public function __construct()
    {
    }

    public function dispatch(Request $request)
    {
        Log::log(LoggingConstants::DEBUG, "Called");

        $config = ORBConfig::getInstance();

        for($i = 0; $i < $request->getBodyCount(); $i++)
        {
            $request->setCurrentBody($i);

            $requestURI = $request->getRequestURI();

            Log::log(LoggingConstants::INFO,
                "requestURI = $requestURI");

            $serviceId = substr($requestURI, 0 , strrpos($requestURI,"."));
            $methodName = substr($requestURI, strlen($serviceId) + 1);

            $arg = $request->getRequestBodyData();

            if(!is_array($arg))
            {
              $arg = array($request->getRequestBodyData());
            }

            Log::log(LoggingConstants::DEBUG,
                "requestURI = $requestURI, "
                . "serviceId = $serviceId, methodName = $methodName, bodyDataType:"
                . gettype($request->getRequestBodyData()) . " className:"
                . get_class($request->getRequestBodyData()));


			try
			{
            	$value = self::handleInvoke($request, $serviceId, $methodName, $arg);

            	$request->setResponseBodyPart($value);
            	$request->setResponseURI("/onResult");
            }
            catch( Exception $e )
            {
            	$request->setResponseBodyPart($e);
            	$request->setResponseURI("/onStatus");
            }
        }

        return true;
    }

    public static function handleInvoke(Request $request, $targetObject, $function, &$arg)
    {
        $config = ORBConfig::getInstance();
        $handlers = $config->getHandlers();
        $resolvedName = ServiceRegistry::getMapping($targetObject);
           
		if( !$config->getSecurity()->canAccess( $resolvedName ) )
		{
			throw new ServiceException( "WebORB security has rejected access to class " . $targetObject . ". see server log or contact system administrator", 401 );
		}

		if( !$config->getSecurity()->canAccess( $resolvedName . "#" . $function ) )
		{
			throw new ServiceException( "WebORB security has rejected access to method " . $targetObject . "." . $function . ". see server log or contact system administrator", 401 );
		}
			
        $value = $handlers->invoke($resolvedName, $function ,$arg);

        return $value->getObject();
    }
}

?>
