<?php
/*******************************************************************
 * V3Dispatcher.php
 * Copyright (C) 2006 Midnight Coders, LLC
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * The software is licensed under the GNU General Public License (GPL)
 * For details, see http://www.gnu.org/licenses/gpl.txt.
 ********************************************************************/


    require_once(WebOrb . "Dispatch/IDispatch.php");
    require_once(WebOrb . "ORBConstants.php");
    require_once(WebOrb . "Reader/NamedObject.php");
    require_once(WebOrb . "V3Types/V3Message.php");
    
    class V3Dispatcher
        implements IDispatch
    {
      
      public function dispatch(Request $request)
      {
        Log::log(LoggingConstants::DEBUG, "Called");
                
        if(!$this->isV3Request($request))
          return false;
        
        $authError = false;
        
        for($i = 0; $i < $request->getBodyCount(); $i++)
        {
          $request->setCurrentBody($i);
          
          $bodyData = $request->getRequestBodyData();
          $namedObject = $bodyData[0];
          $v3Message = $namedObject->defaultAdapt();
      
          if(!$authError)
          {
            $v3Message = $v3Message->execute($request);
            Log::log(LoggingConstants::DEBUG, "Executed " . get_class($v3Message));
          }
          else
          {
            Log::log(LoggingConstants::DEBUG, "Skipped " . get_class($v3Message));
            
            $v3Message = new ErrMessage($v3Message->messageId,
                    new WebORBAuthenticationException("Access is denied"));
          }
          
          $request->setResponseBodyPart($v3Message);
          
          if($v3Message->isError())
          {
            $request->setResponseURI(ORBConstants::ONSTATUS);
            
            if(!$authError)
                $authError = $v3Message->isAuthentication();
          }
          else
            $request->setResponseURI(ORBConstants::ONRESULT);
            
        }
        
        return true;
                
      }
      
      private function isV3Request(Request $request)
      {        
        Log::log(LoggingConstants::DEBUG, "Called");
            
        if(strpos($request->getRequestURI(),"."))
            return false;
            
        if(!is_array($request->getRequestBodyData()))
            return false;
                
         return true;                   
      }
      
      
      public function getName()
      {
        return "V3 Dispatcher";
      }
      
    }

?>