<?php
/*******************************************************************
 * Request.php
 * Copyright (C) 2006 Midnight Coders, LLC
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * The software is licensed under the GNU General Public License (GPL)
 * For details, see http://www.gnu.org/licenses/gpl.txt.
 ********************************************************************/


require_once(WebOrb . "Message/Body.php");
require_once(WebOrb . "Message/Header.php");
require_once(WebOrb . "Writer/IProtocolFormatter.php");

class Request
{
    private $m_version;
    private $m_headers = array();
    private $m_bodyParts = array();
    private $m_currentBodyIndex;
    private $m_responseBodies = array();
    private $m_formatter;

    public function __construct($version, $headers, $bodyParts)
    {
      $this->m_version = $version;
      $this->m_headers = $headers;
      $this->m_bodyParts = $bodyParts;
      $this->m_currentBodyIndex = 0;
    }

    public function getHeader($index)
    {
        return $this->m_header[$index];
    }

    public function getVersion()
    {
        return $this->m_version;
    }

    public function getBodyCount()
    {
      return sizeof($this->m_bodyParts);
    }

    public function getResponseBodies()
    {
      return $this->m_bodyParts;
    }

    public function getResponseHeaders()
    {
      return array();
    }

    public function setCurrentBody($index)
    {
      $this->m_currentBodyIndex = $index;
    }

    public function getRequestURI()
    {
      return $this->currentBody()->getServiceURI();
    }

    public function setResponseBodyPart($object)
    {
      $this->currentBody()->setResponseDataObject($object);
    }

    public function setResponseURI($responseUri)
    {
		$this->currentBody()->setResponseURI(
            $this->currentBody()->getResponseURI() . $responseUri);
	}


    public function setFormatter(IProtocolFormatter $formatter)
    {
	  $this->m_formatter = $formatter;
	}

    public function getFormatter()
    {
	  return $this->m_formatter;
	}

	public function getRequestBodyData()
	{
      return $this->currentBody()->getDataObject();
    }


    private function currentBody()
    {
      return $this->m_bodyParts[$this->m_currentBodyIndex];
    }
}

?>
