<?php
/*******************************************************************
 * ORBHttpHandler.php
 * Copyright (C) 2006 Midnight Coders, LLC
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * The software is licensed under the GNU General Public License (GPL)
 * For details, see http://www.gnu.org/licenses/gpl.txt.
 ********************************************************************/



if (!defined("WebOrb")) {
    define("WebOrb", dirname(__FILE__) . DIRECTORY_SEPARATOR);
}

require_once(WebOrb . "Dispatch/Dispatchers.php");
require_once(WebOrb . "Message/Request.php");
require_once(WebOrb . "Config/ORBConfig.php");
require_once(WebOrb . "Util/Logging/Log.php");
require_once(WebOrb . "Util/Network.php");
require_once(WebOrb . "Writer/MessageWriter.php");

class ORBHttpHandler
{

    public function __construct()
    {
        session_start();
    }

    public function processRequest()
    {
        if($_SERVER["REQUEST_METHOD"] == "GET")
        {
          print("WebORB v1.3.2");
          return;
        }

        $timeStart = microtime(true);

        ob_start();

        $inputData = file_get_contents("php://input");

        try
        {
            $config = ORBConfig::getInstance();

			if (!defined("WebOrbServicesPath"))
			{
				define("WebOrbServicesPath", realpath(WebOrb . $config->getServicePath() ) . '\\');
				Log::log(LoggingConstants::DEBUG, "WebORB services path is - ". WebOrbServicesPath );
			}

            $request = $config->getProtocolRegistry()->buildMessage(
                /*$GLOBALS["HTTP_CONTENT_TYPE"]*/"",
                $inputData); //
        }
        catch(Exception $e)
        {
           Log::logException(LoggingConstants::ERROR,"Internal error",$e);
           return;
        }


        if(Dispatchers::dispatch($request))
        {
            $this->serializeResponse($request);
        }

        $logMessage =  sprintf("Execute time: %0.3f", microtime(true) - $timeStart);

        Log::log(LoggingConstants::PERFORMANCE,$logMessage);
    }

    private function serializeResponse(Request $request)
    {
        Log::log(LoggingConstants::DEBUG, "Called.");

        $formatter = $request->getFormatter();

        MessageWriter::writeObject($request, $formatter);

        $byteBuffer = $formatter->getBytes();

        Log::log(LoggingConstants::INFO,  ob_get_contents());

        ob_clean();

        header("Content-type: " . $formatter->getContentType());
        header("Content-length: " . strlen($byteBuffer));

        $formatter->cleanup();

        print($byteBuffer);

        ob_end_flush();
    }

}

?>
