<?php
/*******************************************************************
 * AnonymousObject.php
 * Copyright (C) 2006 Midnight Coders, LLC
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * The software is licensed under the GNU General Public License (GPL)
 * For details, see http://www.gnu.org/licenses/gpl.txt.
 ********************************************************************/



require_once(WebOrb . "Reader/IAdaptingType.php");
require_once(WebOrb . "Util/Logging/Log.php");

class AnonymousObject
    implements IAdaptingType
{

    private static $c_accessCheckSuppressed = true; // self::canAccessFieldsDirectly();

    private static function canAccessFieldsDirectly()
    {
        /* TODO: Do we need this code in PHP?.. How it can be realized?
        ReflectionPermission permission = new ReflectionPermission( ReflectionPermissionFlag.AllFlags );
        return SecurityManager.IsGranted( permission );
        */
        return true;
    }

    private $m_properties;

    public function __construct(&$properties)
    {
        $this->m_properties = &$properties;
    }

    public function getProperties()
    {
        return $this->m_properties;
    }

    public function getDefaultType()
    {
        return "object";
    }

    public function defaultAdapt()
    {
        $hashtable = array();
        $keys = array_keys($this->m_properties);

        for ($index = 0; $index < count($keys); $index ++)
        {
            $obj = $this->m_properties[$keys[$index]];
            
            Log::log(LoggingConstants::DEBUG, "processing property " . $keys[$index]);

            if ($obj instanceof IAdaptingType)
            {
                $obj = $obj->defaultAdapt();
            }

            $hashtable[$keys[$index]] = $obj;
        }

        return $hashtable;
    }

    public function adapt($type)
    {      
       if($type instanceof ReflectionClass)
            Log::log(LoggingConstants::DEBUG, "Type=" . $type->getName());

        $obj = $type->newInstance();
            
        if (self::$c_accessCheckSuppressed)
        {
            $this->setFieldsDirect($obj, $this->m_properties);
        }
        else
        {
            $this->setFieldsAsBean($obj, $this->m_properties);
        }

        return $obj;
    }

    public function canAdaptTo($formalArg)
    {
        return ($formalArg instanceof ReflectionClass)
            || ("array" == $formalArg);
        /* TODO:
        return typeof( IAdaptingType ).IsAssignableFrom( formalArg ) ||
            (!formalArg.IsArray && !formalArg.IsValueType) ||
            typeof( IDictionary ).IsAssignableFrom( formalArg ) ||
            typeof( StringDictionary ).IsAssignableFrom( formalArg ) ||
            ObjectFactories.GetArgumentObjectFactory( formalArg.FullName ) != null;
        */
    }

    private function setFieldsDirect($obj, $properties)
    {    
        $type = new ReflectionClass(get_class($obj));

        do {
            // Process instance fields

            $fields = $type->getProperties();

            for ($index = 0; $index < count($fields); $index ++)
            {
                if (!isset($this->m_properties[$fields[$index]->getName()]))
                {
                    continue;
                }

                $fieldValue = $this->m_properties[$fields[$index]->getName()];

                /* if (is_null($fieldValue))
                {
                    continue;

                } */

                if($type->getName() == "V3Message" && $fields[$index]->getName() == "body")
                {

                    $body = null;
                  
                    if($fieldValue instanceof ArrayType)
                    {
                      $body = $fieldValue->getArray();
                    } 
                    else
                    {
                      $body = array($fieldValue);
                    }
                    
                    $bodyHolder = new BodyHolder();
                    $bodyHolder->setBody($body);
                    
                    $fieldValue = $bodyHolder;
                    
                }
                else if ($fieldValue instanceof IAdaptingType)
                {
                    /* $val = ObjectFactories::createArgumentObject(
                        $fields[$index]->getDeclaringType()->getName(), $fieldValue);

                    $fieldValue = (!is_null($val))? $val: $fieldValue->adapt($fields[$index]->getDeclaringType()); */
                    if ($fieldValue instanceof NamedObject)
                    {
                        $fieldValue = $fieldValue->adapt($fieldValue->getMappedType());
                    }
                    else
                    {
                        $fieldValue = $fieldValue->defaultAdapt();
                    }
                }
                
                $fields[$index]->setValue($obj, $fieldValue);
            }

            // Process static fields

            $fields = $type->getStaticProperties();

            for ($index = 0; $index < count($fields); $index ++)
            {
                $fieldValue = $this->m_properties[$fields[$index]->getName()];

                if (is_null($fieldValue))
                {
                    continue;
                }

                if ($fieldValue instanceof IAdaptingType)
                {
                    /* $val = ObjectFactories::createArgumentObject(
                        $fields[$index]->getDeclaringType()->getName(), $fieldValue);

                    $fieldValue = (!is_null($val))? $val: $fieldValue->adapt($fields[$index]->getDeclaringType()); */
                    $fieldValue = $fieldValue->adapt($fields[$index]->getDeclaringClass());
                }

                $type->setStaticPropertyValue($fields[$index]->getName(), $fieldValue);
            }

            $type = $type->getParentClass();
        }
        while (is_object($type));
    }

    private function setFieldsAsBean($obj, $properties)
    {

    }

}

?>
