<?php
/*******************************************************************
 * AnonymousObjectReader.php
 * Copyright (C) 2006 Midnight Coders, LLC
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * The software is licensed under the GNU General Public License (GPL)
 * For details, see http://www.gnu.org/licenses/gpl.txt.
 ********************************************************************/



require_once(WebOrb . "Reader/ITypeReader.php");
require_once(WebOrb . "Reader/FlashorbBinaryReader.php");
require_once(WebOrb . "Reader/ParseContext.php");
require_once(WebOrb . "Reader/AnonymousObject.php");
require_once(WebOrb . "Protocols/AMFMessageFactory.php");
require_once(WebOrb . "Util/Datatypes.php");

class AnonymousObjectReader
    implements ITypeReader
{

    public function read(FlashorbBinaryReader $reader, ParseContext $parseContext)
    {
        $properties = array();
        $anonymousObject = new AnonymousObject($properties);
        $parseContext->addReference($anonymousObject);

        while (true)
        {
            $propName = $reader->readUTF();
            $obj = null;

            $dataType = $reader->readByte();

            if (($dataType == Datatypes::REMOTEREFERENCE_DATATYPE_V1) && ($propName != "nc"))
            {
                $obj = 0; // must be an instance of Flash's Number
            }
            else
            {
                $obj = AmfMessageFactory::readData($reader, $parseContext, $dataType);
            }

            if (is_null($obj))
            {
                break;
            }

            $properties[$propName] = $obj;

        }

        return $anonymousObject;
    }

}

?>
