<?php
/*******************************************************************
 * ArrayType.php
 * Copyright (C) 2006 Midnight Coders, LLC
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * The software is licensed under the GNU General Public License (GPL)
 * For details, see http://www.gnu.org/licenses/gpl.txt.
 ********************************************************************/



require_once(WebOrb . "Reader/IAdaptingType.php");
require_once(WebOrb . "Util/Logging/Log.php");
require_once(WebOrb . "Exceptions/ApplicationException.php");

class ArrayType
    implements IAdaptingType
{

    private $m_arrayObject = array();

    public function __construct(&$arrayObject)
    {
        $this->m_arrayObject = &$arrayObject;
    }

    public function getDefaultType()
    {
        return "array";
    }

    public function defaultAdapt()
    {
        $size = sizeof($this->m_arrayObject);

        Log::log(LoggingConstants::DEBUG, "Array size: $size");

        $array = array();

        for ($index = 0; $index < $size; $index ++)
        {
            $obj = $this->m_arrayObject[$index];

            if ($obj instanceof IAdaptingType)
            {
                $obj = $obj->defaultAdapt();
            }

            $array[] = $obj;
        }

        return $array;
    }

    public function adapt($type)
    {
        Log::log(LoggingConstants::DEBUG, "Adapting type: " + $type->getName());

        $size = count($this->arrayObject);

        if ("array" == $type)
        {
            $newArray = array();

            for ($index = 0; $index < $size; $index ++)
            {
                $newArray[$index] = $this->adaptArrayComponent($this->m_arrayObject[$index]);
            }

            return $newArray;
        }
        else if ("IAdaptingType" == $type->getName())
        {
            return $this;
        }
        else
        {
            return $this->defaultAdapt();
        }
    }

    public function canAdaptTo($formalArg)
    {
        return ("array" == $formalArg)
            || ($formalArg instanceof IAdaptingType);
    }

    public function getArray()
    {
        return $this->m_arrayObject;
    }

    private function adaptArrayComponent($obj)
    {
        if ($obj instanceof IAdaptingType)
        {
            return $obj->defaultAdapt();
        }
        else
        {
            throw new ApplicationException( "array element is not adapting type" );
        }
    }

}

?>
