<?php
/*******************************************************************
 * BooleanType.php
 * Copyright (C) 2006 Midnight Coders, LLC
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * The software is licensed under the GNU General Public License (GPL)
 * For details, see http://www.gnu.org/licenses/gpl.txt.
 ********************************************************************/



require_once(WebOrb . "Reader/IAdaptingType.php");
require_once(WebOrb . "Exceptions/ApplicationException.php");

class BooleanType
    implements IAdaptingType
{

    private $m_boolean;

    public function __construct($boolean)
    {
        $this->m_boolean = $boolean;
    }

    public function getDefaultType()
    {
        return "boolean";
    }

    public function defaultAdapt()
    {
        return $this->m_boolean;
    }

    public function adapt($type)
    {
        if ($type instanceof ReflectionClass)
        {
            if ($type->implementsInterface("IAdaptingType"))
            {
                return $this;
            }
        }
        else if ("boolean" == $type)
        {
            return $this->m_boolean;
        }
        else if ("string" == $type)
        {
            return (string) $this->m_boolean;
        }
        else
        {
            throw new ApplicationException("unable to adapt boolean to type " + $type);
        }
    }

    public function canAdaptTo($formalArg)
    {
        if (is_string($formalArg))
        {
            return ("string" == $formalArg)
                || ("boolean" == $formalArg);
        }
        else if ($formalArg instanceof ReflectionClass)
        {
            return $formalArg->implementsInterface("IAdaptingType");
        }
        return FALSE;
    }

}

?>
