<?php
/*******************************************************************
 * NamedObject.php
 * Copyright (C) 2006 Midnight Coders, LLC
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * The software is licensed under the GNU General Public License (GPL)
 * For details, see http://www.gnu.org/licenses/gpl.txt.
 ********************************************************************/



require_once(WebOrb . "Reader/IAdaptingType.php");
require_once(WebOrb . "Util/Logging/Log.php");
require_once(WebOrb . "Util/TypeLoader.php");
require_once(WebOrb . "Registry/ServiceRegistry.php");
require_once(WebOrb . "Types/Types.php");

class NamedObject
    implements IAdaptingType
{

    private $m_objectName;
    private $m_typedObject;
    private $m_mappedType;

    public function __construct($objectName, IAdaptingType $typedObject)
    {
        Log::log(LoggingConstants::DEBUG, $objectName);

        $this->m_objectName = $objectName;
        $this->m_typedObject = $typedObject;

        $this->m_mappedType = Types::getServerTypeForClientClass($objectName);

    }

    public function getDefaultType()
    {
        if (!is_null($this->m_mappedType))
        {
            return $this->m_mappedType;
        }
        return $this->m_typedObject->getDefaultType();
    }

    public function defaultAdapt()
    {

        if (!is_null($this->m_mappedType))
        {
            return $this->m_typedObject->adapt($this->m_mappedType);
        }
        return $this->m_typedObject->defaultAdapt();
    }

    public function adapt($type)
    {
        if (!is_null($this->m_mappedType) && $this->m_mappedType->isSubclassOf($type))
        {
            return $this->m_typedObject->adapt($this->m_mappedType);
        }
        return $this->m_typedObject->adapt($type);
    }

    public function canAdaptTo($formalArg)
    {
        if (!is_null($this->m_mappedType))
        {
            return ((($formalArg->isInterface() || $formalArg->isAbstract())
                && $this->m_mappedType->isSubclassOf($formalArg)))
                || $formalArg->isSubclassOf($this->m_mappedType);
        }

        if ($formalArg->getName() == $this->m_objectName)
        {
            return true;
        }

        return false;
    }

    public function getMappedType()
    {
    	return $this->m_mappedType;
    }
}

?>
