<?php
/*******************************************************************
 * NumberObject.php
 * Copyright (C) 2006 Midnight Coders, LLC
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * The software is licensed under the GNU General Public License (GPL)
 * For details, see http://www.gnu.org/licenses/gpl.txt.
 ********************************************************************/



require_once(WebOrb . "Reader/IAdaptingType.php");

class NumberObject
    implements IAdaptingType
{

    private $m_data;

    public function __construct($data)
    {
        $this->m_data = $data;
    }

    public function getDefaultType()
    {
        return "integer";
    }

    public function defaultAdapt()
    {
        return $this->m_data;
    }

    public function adapt($type)
    {
        if ("string" == $type)
        {
            return (string) $this->m_data;
        }
        else if ("integer" == $type)
        {
            return (integer) $this->m_data;
        }
        else if ("float" == $type)
        {
            return (float) $this->m_data;
        }
        else if ("boolean" == $type)
        {
            return (boolean) $this->m_data;
        }
        else if ($type instanceof ReflectionClass)
        {
            if ($type->implementsInterface("IAdaptingType")) {
                return $this;
            }
        }
        return $this->m_data;
    }

    public function canAdaptTo($formalArg)
    {
        if (is_string($formalArg))
        {
            return ("string" == $formalArg)
                || ("float" == $formalArg)
                || ("boolean" == $formalArg)
                || ("integer" == $formalArg);
        }
        else if ($formalArg instanceof ReflectionClass)
        {
            return $formalArg->implementsInterface("IAdaptingType");
        }
        return false;
    }

}

?>
