<?php
/*******************************************************************
 * StringType.php
 * Copyright (C) 2006 Midnight Coders, LLC
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * The software is licensed under the GNU General Public License (GPL)
 * For details, see http://www.gnu.org/licenses/gpl.txt.
 ********************************************************************/



require_once(WebOrb . "Exceptions/ApplicationException.php");
require_once(WebOrb . "Reader/IAdaptingType.php");
require_once(WebOrb . "Util/Logging/Log.php");

class StringType
    implements IAdaptingType
{

    private $m_stringValue;

    public function __construct($stringValue)
    {
        $this->m_stringValue = $stringValue;
    }

    public function getDefaultType()
    {
        return "string";
    }

    public function defaultAdapt()
    {
        return $this->m_stringValue;
    }

    public function adapt($type)
    {
        if ($type instanceof ReflectionClass)
        {
            if ($type->implementsInterface("IAdaptingType")) {
                return $this;
            }
        }
        else if ("string" == $type)
        {
            return $this->m_stringValue;
        }
        else if (("boolean" == $type) && $this->isBooleanString())
        {
            return (boolean) $this->m_stringValue;
        }
        else if ($type instanceof ReflectionClass)
        {
            if ($type->implementsInterface("IAdaptingType")) {
                return $this;
            }
        }
        else if ("string" == $type)
        {
            return $this->m_stringValue;
        }
        else
        {
            throw new ApplicationException("unable to adapt string to " + $type);
        }
    }

    public function canAdaptTo($formalArg)
    {
        if (is_string($formalArg))
        {
            return ("string" == $formalArg)
                || ("boolean" == $formalArg) && $this->isBooleanString();
        }
        else if ($formalArg instanceof ReflectionClass)
        {
            return $formalArg->implementsInterface("IAdaptingType");
        }
        return FALSE;
    }

    private function isBooleanString()
    {
        $testStr = strtolower($this->m_stringValue);
        return "false" == $testStr
            || "true" == $testStr
            || "yes" == $testStr
            || "no" == $testStr
            || "0" == $testStr
            || "1" == $testStr;
    }

}

?>
