<?php
/*******************************************************************
 * V3ArrayReader.php
 * Copyright (C) 2006 Midnight Coders, LLC
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * The software is licensed under the GNU General Public License (GPL)
 * For details, see http://www.gnu.org/licenses/gpl.txt.
 ********************************************************************/



require_once(WebOrb . "Reader/ITypeReader.php");
require_once(WebOrb . "Reader/FlashorbBinaryReader.php");
require_once(WebOrb . "Reader/ParseContext.php");
require_once(WebOrb . "Reader/ReaderUtils.php");
require_once(WebOrb . "Reader/AnonymousObject.php");
require_once(WebOrb . "Protocols/AMFMessageFactory.php");

class V3ArrayReader
    implements ITypeReader
{

    public function read(FlashorbBinaryReader $reader, ParseContext $parseContext)
    {
        $refId = $reader->readVarInteger();

        if (($refId & 0x1) == 0)
        {
            return $parseContext->getReference($refId >> 1);
        }

        $arraySize = $refId >> 1;
        $adaptingType = null;
        $container = null;

        while (true)
        {
            $str = ReaderUtils::readString($reader, $parseContext);

            if (is_null($str) || strlen($str) == 0)
            {
                break;
            }

            if (is_null($container))
            {
                $container = array();
                $adaptingType = new AnonymousObject($container);
                $parseContext->addReference($adaptingType);
            }

            $container[$str] = AmfMessageFactory::readData($reader, $parseContext, null);
        }

        if (is_null($adaptingType))
        {
            $container = array();
            $adaptingType = new ArrayType($container);
            $parseContext->addReference($adaptingType);

            for ($i = 0; $i < $arraySize; $i ++)
            {
                $container[$i] = AmfMessageFactory::readData($reader, $parseContext, null);
            }
        }
        else
        {
            for ($i = 0; $i < $arraySize; $i ++)
            {
                $obj = AmfMessageFactory::readData($reader, $parseContext, null);
                $container[(string) $i] = $obj;
            }
        }

        return $adaptingType;
    }

}

?>
