<?php
/*******************************************************************
 * V3ObjectReader.php
 * Copyright (C) 2006 Midnight Coders, LLC
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * The software is licensed under the GNU General Public License (GPL)
 * For details, see http://www.gnu.org/licenses/gpl.txt.
 ********************************************************************/



require_once(WebOrb . "Reader/ITypeReader.php");
require_once(WebOrb . "Reader/FlashorbBinaryReader.php");
require_once(WebOrb . "Reader/ParseContext.php");
require_once(WebOrb . "Reader/ClassInfo.php");
require_once(WebOrb . "Reader/IAdaptingType.php");
require_once(WebOrb . "Reader/NamedObject.php");
require_once(WebOrb . "Reader/AnonymousObject.php");
require_once(WebOrb . "Reader/ReaderUtils.php");
require_once(WebOrb . "Protocols/AMFMessageFactory.php");

class V3ObjectReader
    implements ITypeReader
{

    public function read(FlashorbBinaryReader $reader, ParseContext $parseContext)
    {
        $refId = $reader->readVarInteger();

        if (($refId & 0x1) == 0)
        {
            return $parseContext->getReference($refId >> 1);
        }

        $classInfo = $this->getClassInfo($refId, $reader, $parseContext);


        $props = array();
        $obj = new AnonymousObject($props);
        $returnValue = $obj;

        if (!is_null($classInfo->getClassName()) && strlen($classInfo->getClassName()) > 0)
        {
            $returnValue = new NamedObject($classInfo->getClassName(), $obj);
        }

        $parseContext->addReference($returnValue);
        $propCount = $classInfo->getPropertyCount();

        for ($i = 0; $i < $propCount; $i ++)
        {
            $props[$classInfo->getProperty($i)] = AmfMessageFactory::readData($reader, $parseContext, null);
        }

        if ($classInfo->getLooseProps())
        {
            while (TRUE)
            {
                $propName = ReaderUtils::readString($reader, $parseContext);

                if (is_null($propName) || strlen($propName) == 0)
                {
                    break;
                }

                $props[$propName] = AmfMessageFactory::readData($reader, $parseContext, null);
            }
        }

        return $returnValue;
    }

    private function getClassInfo($refId, FlashorbBinaryReader $reader, ParseContext $parseContext)
    {
        if (($refId & 0x3) == 1)
        {
            return $parseContext->getClassInfoReference($refId >> 2);
        }

        $looseProps = ($refId & 0x8) == 8;
        $className = ReaderUtils::readString($reader, $parseContext);
        $classInfo = new ClassInfo($looseProps, $className);

        $propsCount = $refId >> 4;

        for ($i = 0; $i < $propsCount; $i ++)
        {
            $classInfo->addProperty(ReaderUtils::readString($reader, $parseContext));
        }

        $parseContext->addClassInfoReference($classInfo);

        return $classInfo;
    }

}

?>
