<?php
/*******************************************************************
 * V3XmlReader.php
 * Copyright (C) 2006 Midnight Coders, LLC
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * The software is licensed under the GNU General Public License (GPL)
 * For details, see http://www.gnu.org/licenses/gpl.txt.
 ********************************************************************/



require_once(WebOrb . "Reader/ITypeReader.php");
require_once(WebOrb . "Reader/FlashorbBinaryReader.php");
require_once(WebOrb . "Reader/ParseContext.php");
require_once(WebOrb . "Reader/XmlDataType.php");

class V3XmlReader
    implements ITypeReader
{

    public function read(FlashorbBinaryReader $reader, ParseContext $parseContext)
    { 
        $len = $reader->readVarInteger();

        if (($len & 0x1) == 0)
        {
            return $parseContext->getReference($len >> 1);
        }

        $len = $len >> 1;

        if ($len == 0)
        {
            return $this->parseString("");
        }

        $xmlStr = $reader->readUTF($len);
        $xmlType = $this->parseString($xmlStr);
        $parseContext->addReference($xmlType);
        return $xmlType;
    }

    private function parseString($xmlStr)
    {
        $document = new DomDocument();

        try
        {
            $document->loadXml($xmlStr);
        }
        catch (Exception $ex)
        {
        }

        return new XmlDataType($document);
    }

}

?>
