<?php
/*******************************************************************
 * ServiceRegistry.php
 * Copyright (C) 2006 Midnight Coders, LLC
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * The software is licensed under the GNU General Public License (GPL)
 * For details, see http://www.gnu.org/licenses/gpl.txt.
 ********************************************************************/



require_once(WebOrb . "Config/ORBConfig.php");

class ServiceRegistry
{

    public static function getMapping($name)
    {
        return ORBConfig::getInstance()->getServiceRegistry()->_getMapping($name);
    }

    public static function getReverseMapping($mappedName)
    {
        return ORBConfig::getInstance()->getServiceRegistry()->_getReverseMapping($mappedName);
    }

    public static function addMapping($name, $mappedName, $context = null)
    {
        ORBConfig::getInstance()->getServiceRegistry()->_addMapping(
            $name, $mappedName, (is_null($context))? array(): $context);
    }

    public static function removeMapping($name)
    {
        ORBConfig::getInstance()->getServiceRegistry()->_removeMapping($name);
    }

    public static function getContext($type)
    {
        return ORBConfig::getInstance()->getServiceRegistry()->_getContext($type);
    }

    private $m_namedServices;
    private $m_reversedMapping;
    private $m_contexts;

    public function __construct()
    {
        $this->m_namedServices = array();
        $this->m_reversedMapping = array();
        $this->m_contexts = array();
    }

    public function _getMapping($name)
    {
        if (isset($this->m_namedServices[$name]))
        {
            return $this->m_namedServices[$name];
        }
        return $name;
    }

    public function _getReverseMapping($mappedName)
    {
        if (isset($this->m_reversedMapping[$mappedName]))
        {
            return $this->m_reversedMapping[$mappedName];
        }
        return $mappedName;
    }

    public function _addMapping($name, $mappedName, $context)
    {
        $this->m_namedServices[$name] = $mappedName;
        $this->m_reversedMapping[$mappedName] = $name;
        $this->m_contexts[$mappedName] = $context;
    }

    public function _removeMapping($name)
    {
        if (isset($this->m_namedServices[$name]))
        {
            unset($this->m_namedServices[$name]);
            unset($this->m_reversedMapping[$this->m_namedServices[$name]]);
        }
    }

    public function _getContext($type)
    {
        return $this->m_contexts[$type];
    }

}

?>
