<?php
/*******************************************************************
 * AccessConstraint.php
 * Copyright (C) 2006 Midnight Coders, LLC
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * The software is licensed under the GNU General Public License (GPL)
 * For details, see http://www.gnu.org/licenses/gpl.txt.
 ********************************************************************/
 
 require_once(WebOrb . "Exceptions/ConfigurationException.php");
 require_once(WebOrb . "Security/RoleNameRestriction.php");
 require_once(WebOrb . "Security/HostNameRestriction.php");
 require_once(WebOrb . "Security/SingleIPRestriction.php");
 require_once(WebOrb . "Security/IPRangeRestriction.php");
 
    class AccessConstraint
    {
 
        const GRANT_ACTION_NAME = "grant";
        const REJECT_ACTION_NAME = "reject";
        const GRANT_ACTION = 1;
        const REJECT_ACTION = 0;
        
        public $name;
        public $action;
        
        public $restrictions = array();
        
        public function __construct($name,$action)
        {
          $this->name = $name;
          
          $actionName = strtolower($action);
          
          if($actionName == AccessConstraint::GRANT_ACTION_NAME)
            $this->action = AccessConstraint::GRANT_ACTION;
          else if($actionName == AccessConstraint::REJECT_ACTION_NAME)
            $this->action = AccessConstraint::REJECT_ACTION;
          else
            throw new ConfigurationException( "invalid access constraint action. expected 'grant' or 'reject', received " + $action );
            
        }
        
        public function getName()
        {
          return $this->name;
        }
        
        public /*string*/ function getAction()
        {
          return $this->action == 1 ? AccessConstraint::GRANT_ACTION_NAME : AccessConstraint::REJECT_ACTION_NAME;
        }
        
        public /*array*/ function getRestrictions()
        {
          return $this->restrictions;
        }
        
        public /*bool*/ function validate()
        {
          foreach($this->restrictions as $restriction)
          {
            if($restriction->pass())
            { 
              if($this->action == AccessConstraint::REJECT_ACTION)
                return false;
            }
            else
            {      
              if($this->action == AccessConstraint::GRANT_ACTION)
                return false;
            }
          }
          
          return true;
        }
        
        public /*string*/ function getReason()
        {
          foreach($this->restrictions as $restriction)
          {
            if($restriction->pass())
            {
              if($this->action == AccessConstraint::REJECT_ACTION)
                return $restriction->getDetails(false);
            }
            else
            {
              if($this->action == AccessConstraint::GRANT_ACTION)
                return $restriction->getDetails(true);
            }
          }  
          
          return "";
        }
        
        
        public /*void*/ function removeRestriction(IRestriction $restriction)
        {
            if (($i = array_search($restriction, $this->restrictions)) !== false)
                unset($this->restrictions[$i]);
        }
        
        public /*void*/ function addRestriction(IRestriction $restriction)
        {
          $this->restrictions[] = $restriction;
        }
        
        public /*void*/ function addRole($roleName)
        {
          $this->restrictions[] = new RoleNameRestriction($roleName);
        }
        
        public /*void*/ function addHostName($hostName)
        {
          $this->restrictions[] = new HostNameRestriction($hostName);
        }
        
        public /*void*/ function addIP($ipAddress)
        {
          $this->restrictions[] = new SingleIPRestriction($ipAddress);
        }
        
        public /*void*/ function addIPRange($subnetAddress, $subnetMask)
        {
          $this->restrictions[] = new IPRangeRestriction($ipAddress);
        }
    }
 
 ?>