<?php
/*******************************************************************
 * SingleIPRestriction.php
 * Copyright (C) 2006 Midnight Coders, LLC
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * The software is licensed under the GNU General Public License (GPL)
 * For details, see http://www.gnu.org/licenses/gpl.txt.
 ********************************************************************/
 
 require_once(WebOrb . "Security/IRestriction.php");
 require_once(WebOrb . "Exceptions/ConfigurationException.php");
 require_once(WebOrb . "Util/Network.php");
   
 class SingleIPRestriction
    implements IRestriction
 {
    private $m_ipAddressTokens = array();
    
	private $m_isMask;
	private $m_ipAddress;
	
	public function __construct( $ipAddress )
	{
	  $this->m_ipAddress = $ipAddress;
	  $tokens = explode(".", $ipAddress); 

	  if(count($tokens) > 4 )
		 throw new ConfigurationException( "invalid ip address - cannot parse. " + $ipAddress );

	  foreach($tokens as $token)
	  {
		$comparator = null;

		if($token == "*")
		{
			$comparator = AnyTokenComparator::getInstance();
			$this->m_isMask = true;
		}
		else
			$comparator = new TokenComparator( $token );

		$this->m_ipAddressTokens[] = $comparator;
		
      }	
	}
     
	public function pass()
	{
	  $remoteAddr = Network::getUserHostAddress();
	  $tokens = explode(".",$remoteAddr);
	  
      $i = 0;   
      
      foreach($this->m_ipAddressTokens as $tokenComparator)
      {
        if(!$tokenComparator->match($tokens[$i++]))
            return false;
      }
      
	  return true;
	}  
    
    public function getDetails($grant)
    {
      $remoteAddr = Network::getUserHostAddress();
      $details = "Remote host address ";
      $details.= $remoteAddr;
      
      if($grant)
        $details .= " does not match ";
      else
        $details .= " matches ";
      
	  if($this->m_isMask )
		$details .= " the mask of a IP address: ";
	  else
		$details .= " the IP address: ";      
	
	  $details .= $this->m_ipAddress;
	  $details .= ". access to this address has been ";	  
	  
	  if($grant)
	   $details .= " granted";
	  else
	   $details .= " rejected";
	   
	   return $details;
    }
  
    public function isMask()
    {
      return $this->m_isMask;
    }
    
    public function ipAddress()
    {
      return $this->m_ipAddress;
    }
    
 }
?>