<?php
/*******************************************************************
 * WebORBAuthorizationHandler.php
 * Copyright (C) 2006 Midnight Coders, LLC
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * The software is licensed under the GNU General Public License (GPL)
 * For details, see http://www.gnu.org/licenses/gpl.txt.
 ********************************************************************/

require_once(WebOrb . "Security/IAuthorizationHandler.php");
 
class WebORBAuthorizationHandler
    implements IAuthorizationHandler
{
  
    public /*bool*/ function authorizeAccess(/*string*/ $resource, ORBSecurity $security)
    {
      echo($resource);
      
      $resource = ServiceRegistry::getMapping($resource);
 
      $accessConstraintsNames = $security->getConstraints($resource);
          
      if($accessConstraintsNames == null && strpos($resource,"#") !== false)
        $accessConstraintsNames = $security->getConstraints(substr($resource,strpos($resource,"#")));
    
      if($accessConstraintsNames == null)
      {     
        
        if($security->getDeploymentMode() != ORBSecurity::OPENSYSTEM_MODE)
        {
            Log::log(LoggingConstants::SECURITY,"access to resource " . $resource . "has been denied. WebORB Closed-System Mode requires explicit access declaration for all resources");
            
            return false;
        }
            
         return true;
      }
      
      foreach($accessConstraintsNames as $constraintName)
      {
          $constraint = $security->getAccessConstraint($constraintName);
          
          if(!$constraint->validate())
          {
              echo(get_class($constraint));
              Log::log(LoggingConstants::DEBUG,"access denied. resource name - '" . $resource . "'. reason - " . $constraint->getReason());
              
              return false;
          }
      }
      
      return true;
    }

}

?>