<?php
/*******************************************************************
 * DatabaseReaderFactory.php
 * Copyright (C) 2006 Midnight Coders, LLC
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * The software is licensed under the GNU General Public License (GPL)
 * For details, see http://www.gnu.org/licenses/gpl.txt.
 ********************************************************************/
 
 require_once(WebOrb . "Util/DatabaseReaders/MsSqlDatabaseReader.php");
 require_once(WebOrb . "Util/DatabaseReaders/MySqlDatabaseReader.php");
   
 class DatabaseReaderFactory
 {
    private $m_readers;
   
    private function __construct()
    {
        $this->m_readers = array();
        
        $this->m_readers['mssql'] = new MsSqlDatabaseReader();
        $this->m_readers['mysql'] = new MySqlDatabaseReader();        
    }
   
   
    private static $s_instance = null;
    
    private static function getInstance()
    {
        if(self::$s_instance == null)
            self::$s_instance = new DatabaseReaderFactory();
        
        return self::$s_instance;
    }
     
    public static function getReader($databaseName)  
    {
        if(array_key_exists($databaseName,self::getInstance()->m_readers))
            return self::getInstance()->m_readers[$databaseName];
            
        return null;
    }
 }
 
?>