<?php
/*******************************************************************
 * Invocation.php
 * Copyright (C) 2006 Midnight Coders, LLC
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * The software is licensed under the GNU General Public License (GPL)
 * For details, see http://www.gnu.org/licenses/gpl.txt.
 ********************************************************************/



require_once(WebOrb . "Util/Logging/Log.php");
require_once(WebOrb . "Util/ThreadContext.php");
require_once(WebOrb . "Exceptions/WebORBAuthenticationException.php");

class Invocation
{

    static public function invoke($object, ReflectionMethod $method, $arguments)
    {
        Log::log(LoggingConstants::INFO, "method = $method");

        $parameters = $method->getParameters();

        $args = array();

        for($i = 0; $i < sizeof($parameters); $i++)
        {
            $arg = $arguments[$i];

            Log::log(LoggingConstants::DEBUG, "type: " . get_class($arg));

            if (is_null($parameters[$i]->getClass()))
            {
                if ($arg instanceof IAdaptingType)
                    $args[] = $arg->defaultAdapt();
                else
                    $args[] = $arg;
            }
            else
            {
                if ($arg instanceof IAdaptingType)
                    $args[] = $arg->adapt($parameters[$i]->getClass());
                else
                    $args[] = $arg;
            }
        }



        try
        {
		    $class = new ReflectionClass( 'ReflectionMethod' );
 		    $invokeArgsMethod = $class->getMethod( 'invokeArgs' );
        }
        catch( Exception $e )
        {
			if( sizeof( $args ) == 1 )
			   $args = $args[ 0 ];

			$result = $method->invoke($object, $args);

			if( is_array( $args ) && sizeof($parameters) > 1 )
			   $result = $result[ 0 ];

			return $result;
        }

        return $method->invokeArgs($object, $args);

    }

}

?>
