<?php
/*******************************************************************
 * Log.php
 * Copyright (C) 2006 Midnight Coders, LLC
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * The software is licensed under the GNU General Public License (GPL)
 * For details, see http://www.gnu.org/licenses/gpl.txt.
 ********************************************************************/



require_once(WebOrb . "Util/Logging/LoggingConstants.php");

class Log
{

    private function __construct(){}

    static private $s_handle = null;

    static private function getHandle()
    {
        if (is_null(self::$s_handle))
        {
            self::$s_handle = fopen(WebOrb."weborb-log.txt", "a");
        }
        return self::$s_handle;
    }


    public static function isChannelAllowed($messageType)
    {
      $filter = LoggingConstants::DEBUG | LoggingConstants::PERFORMANCE | LoggingConstants::ERROR | LoggingConstants::EXCEPTION;

      return ($filter & $messageType) != 0;
    }

    public static function logException($messageType, $message, Exception $exception)
    {
       if(!self::isChannelAllowed($messageType))
            return;

        $line = date("c");
        $line .= " [" . LoggingConstants::getLogChannelName($messageType) . "] ";
        $line .= self::getSource();
        $line .= $message . "\r\n";
		/* $line .= get_class($exception);
		$line .= " thrown with message: " . $exception->getCode() . "\"";
		$line .= $exception->getMessage() . "\" at " . $exception->getFile();
		$line .= ":" . $exception->getLine() . "\r\n";
		$line .= $exception->getTraceAsString(); */
		$line .= $exception->__toString() . "\r\n";

        $source = debug_backtrace();

        fwrite(self::getHandle(), $line);
    }


    public static function log($messageType, $message)
    {

       if(!self::isChannelAllowed($messageType))
            return;

        $line = date("c");
        $line .= " [" . LoggingConstants::getLogChannelName($messageType) . "] ";
        $line .= self::getSource();
        $line .= $message . "\r\n";

        $source = debug_backtrace();

        fwrite(self::getHandle(), $line);
    }

    public static function getSource()
    {
        $bt = debug_backtrace();

        if (isset($bt[2]))
        {
            $class = $bt[2]['class'];
            $function = $bt[2]['function'];

            $file = $bt[1]['file'];
            $line = $bt[1]['line'];

            // return "$class::$function: in $file at $line";
            return "$class::$function:";
        }
        else
        {
            return "";
        }
    }

    public function __destruct()
    {
        fclose(self::getHandle());
        self::$s_handle = null;
    }

}

?>
