<?php
/*******************************************************************
 * Network.php
 * Copyright (C) 2006 Midnight Coders, LLC
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * The software is licensed under the GNU General Public License (GPL)
 * For details, see http://www.gnu.org/licenses/gpl.txt.
 ********************************************************************/

 class Network
 {
  private static $s_privateIPArray=array(
                       "10.0.0.0/8",
                       "172.16.0.0/12",
                       "192.168.0.0/16");

  public static function getUserIPArray()
  {
    $ipList = array();
    
    if(isset($_SERVER['HTTP_X_FORWARDED_FOR'])) 
    {
      if(strpos($_SERVER['HTTP_X_FORWARDED_FOR'],',') !== false)
       $ipList +=  explode(',',$_SERVER['HTTP_X_FORWARDED_FOR']);
      else
       $ipList[] = $_SERVER['HTTP_X_FORWARDED_FOR'];    
    }
    
    $ipList[] = $_SERVER['REMOTE_ADDR'];
    
    return $ipList;      
  }

  public static function isIPInNet($ip,$net,$mask) 
  {
    $lnet=ip2long($net);
    $lip=ip2long($ip);
    $binnet=str_pad( decbin($lnet),32,"0","STR_PAD_LEFT" );
    $firstpart=substr($binnet,0,$mask);
    $binip=str_pad( decbin($lip),32,"0","STR_PAD_LEFT" );
    $firstip=substr($binip,0,$mask);
                                                                               
    return(strcmp($firstpart,$firstip)==0);
 }
 
 public static function isIpInNetArray($ip,&$array)
 {
  $returnValue = false;
  
  foreach ( $array as $subnet ) 
  {
   list($net,$mask)=split("/",$subnet);
   
   if(Network::isIPInNet($ip,$net,$mask))
   {
     $returnValue = true;
     break;
   }
  }
  return $returnValue;
 }
 
 public static function isIpInPrivateNetwork($ip)
 {
    return self::isIpInNetArray($ip,self::$s_privateIPArray);  
 }
 
 public static function getUserHostAddress()
 {
   $ip = "unknown";
   $ip_array = self::getUserIPArray();
   
   foreach ( $ip_array as $ip_s ) 
   {                                                           
     if( $ip_s !="")
        $ip = $ip_s;
     else
        continue;
    
     if(!Network::isIPInNetArray($ip_s,self::$s_privateIPArray))
       break;
   }
                                                                                                                                                               
    return $ip;
  } 
  
  public static function getUserHostName()
  {
    $remouteHost = $_SERVER['REMOTE_HOST'];
    
    if($remouteHost == null || strlen($remouteHost) == 0)
        $remouteHost = gethostbyaddr(self::getUserHostAddress());
        
    return $remouteHost;
  }

  public static function getServerIp()
  {

    return gethostbyname($_SERVER["SERVER_NAME"]);
  }
  
  public static function getServerName()
  {

    return $_SERVER["SERVER_NAME"];
  }
  
  public static function getLocalIp()
  {
    return gethostbyname("localhost");    
  }
  
}