<?php
/*******************************************************************
 * TypeLoader.php
 * Copyright (C) 2006 Midnight Coders, LLC
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * The software is licensed under the GNU General Public License (GPL)
 * For details, see http://www.gnu.org/licenses/gpl.txt.
 ********************************************************************/



require_once(WebOrb . "Config/ORBConfig.php");
require_once(WebOrb . "Util/Logging/Log.php");
require_once(WebOrb . "Reader/Dataset/DataSetInfo.php");
require_once(WebOrb . "V3Types/ReqMessage.php");
require_once(WebOrb . "V3Types/AckMessage.php");
require_once(WebOrb . "V3Types/DataMessage.php");
require_once(WebOrb . "V3Types/PagedMessage.php");
require_once(WebOrb . "V3Types/SeqMessage.php");
require_once(WebOrb . "V3Types/ErrMessage.php");
require_once(WebOrb . "V3Types/ErrDataMessage.php");
require_once(WebOrb . "Security/WebORBAuthorizationHandler.php");
require_once(WebOrb . "Security/WebORBAuthenticationHandler.php");

class TypeLoader
{

    static public function loadType($typeName)
    {
        if($typeName == "Weborb.V3Types.ReqMessage")
        {
          return new ReflectionClass("ReqMessage");
        }
        else if($typeName == "Weborb.V3Types.AckMessage")
        {
          return new ReflectionClass("AckMessage");
        }
        else if($typeName == "Weborb.V3Types.DataMessage")
        {
          return new ReflectionClass("DataMessage");
        }
        else if($typeName == "Weborb.V3Types.PagedMessage")
        {
          return new ReflectionClass("PagedMessage");
        }
        else if($typeName == "Weborb.V3Types.SeqMessage")
        {
          return new ReflectionClass("SeqMessage");
        }
        else if($typeName == "Weborb.V3Types.ErrMessage")
        {
          return new ReflectionClass("ErrMessage");
        }
        else if($typeName == "Weborb.V3Types.ErrDataMessage")
        {
          return new ReflectionClass("ErrDataMessage");
        }
        else if($typeName == "Weborb.Reader.Dataset.DataSetInfo")
        {
          return new ReflectionClass("DataSetInfo");
        }
        else if($typeName == "Weborb.Security.WebORBAuthorizationHandler")
        {
          return new ReflectionClass("WebORBAuthorizationHandler");
        }
        else if($typeName == "Weborb.Security.WebORBAuthenticationHandler")
        {
          return new ReflectionClass("WebORBAuthenticationHandler");
        }
        
        Log::log(LoggingConstants::DEBUG, "Called. typeName = $typeName");

        $config = ORBConfig::getInstance();
        $servicePath = $config->getServicePath();

        Log::log(LoggingConstants::DEBUG, "What is weborb " . WebOrb);

        $prefix = realpath(WebOrb . $servicePath) . DIRECTORY_SEPARATOR;

        Log::log(LoggingConstants::DEBUG, "Services directory: " . $prefix);

        $path = $prefix . str_replace(".", DIRECTORY_SEPARATOR, $typeName) . '.php';

        Log::log(LoggingConstants::DEBUG, "Service path: " . $path);

        if (!file_exists($path))
        {
            $path = $prefix . str_replace(
                ".", DIRECTORY_SEPARATOR, $config->getServiceRegistry($typeName)->getMapping($typeName)) . '.php';
        }

		Log::log(LoggingConstants::DEBUG, "found typeName " . $path);
		Log::log(LoggingConstants::DEBUG, "check");

try
{
        require_once($path);
}
catch (Exception $ex)
{
	 Log::logException(LoggingConstants::DEBUG, "Something wrong...", $ex);
}
        Log::log(LoggingConstants::DEBUG, "still here");

        $pieces = explode('.', $typeName);

        Log::log(LoggingConstants::DEBUG, "pieces " . $pieces );

        $class = new ReflectionClass($pieces[count($pieces) - 1]);
        return $class;
    }

    static public function getFullClassName(ReflectionClass $class)
    {
      $config = ORBConfig::getInstance();

      $servicePath = realpath(WebOrb . $config->getServicePath());

      $fileName = $class->getFileName();

      $excludingServicePath = substr($fileName,strlen($servicePath)+1);
      //$excludingServicePath = substr($excludingServicePath,strlen($servicePath)+1);
      $excludingServicePath = substr($excludingServicePath,0,strlen($excludingServicePath)-4); // removing .php
      $excludingServicePath = str_replace(DIRECTORY_SEPARATOR, '.', $excludingServicePath);

      Log::log(LoggingConstants::DEBUG, $excludingServicePath);

      return $excludingServicePath;
    }

}

?>
