<?php
/*******************************************************************
 * ReqMessage.php
 * Copyright (C) 2006 Midnight Coders, LLC
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * The software is licensed under the GNU General Public License (GPL)
 * For details, see http://www.gnu.org/licenses/gpl.txt.
 ********************************************************************/

    require_once(WebOrb . "V3Types/V3Message.php");
    require_once(WebOrb . "Util/ThreadContext.php");
    require_once(WebOrb . "V3Types/ErrMessage.php");    
    require_once(WebOrb . "Security/ORBSecurity.php");  
    require_once(WebOrb . "Security/Credentials.php");    
        
    class ReqMessage
        extends V3Message
    {
      public $operation;
      public $source;
      public $messageRefType;
      
      const AUTHENTICATION_MESSAGE_REF_TYPE = "flex.messaging.messages.AuthenticationMessage";
      
      const CLIENT_PING_OPERATION = 5;
      const CLIENT_SYNC_OPERATION = 4;
      const CLUSTER_REQUEST_OPERATION = 7;
      const LOGIN_OPERATION = 8;
      const LOGOUT_OPERATION = 9;
      const POLL_OPERATION = 2;
      const SESSION_INVALIDATE_OPERATION = 10;
      const SUBSCRIBE_OPERATION = 0;
      const UNKNOWN_OPERATION = 1000;
      const UNSUBSCRIBE_OPERATION = 1;
    
    
      public function setOperation($operation)
      {
        $this->operation = $operation;
      }

      public function getOperation()
      {
        return $this->operation;
      }

	  public function setSource($source)
	  {
  	    $this->source = $source;
	  }

	  public function getSource()
	  {
	    return $this->source;
	  }

      public function setMessageRefType($value)
      {
        $this->messageRefType = $value;
      }

      public function getMessageRefType()
      {
        return $this->messageRefType;
      }
      
      public function isAuthenticateMessage()
      {
        return $this->body->getBody() == ReqMessage::AUTHENTICATION_MESSAGE_REF_TYPE;
      }

      public function execute(Request $request)
      {

        if ("5" == $this->operation || "2" == $this->operation || "0" == $this->operation)
        {
            return new AckMessage($this->messageId, $this->clientId, null);
        }
        else if( "9" == $this->operation )
        {
            ThreadContext::setCallerCredentials( null );
            
            return new AckMessage($this->messageId, $this->clientId, null);
        }
        else if("8" == $this->operation)
        {
            $arr = $this->body->getBody();
            $adaptingType = $arr[0];
            
            $authData = split(":", base64_decode($adaptingType->defaultAdapt()));
            
            $credentials = new Credentials($authData[0],$authData[1]);   
            
            $authHandler = ORBSecurity::getAuthenticationHandler( ThreadContext::getORBConfig() );
            
            if( $authHandler == null )
            {
                $errorMessage = new ErrMessage( $this->messageId, new ServiceException( "Missing authentication handler" ) );
                $errorMessage->faultCode = "Client.Authentication";
                return $errorMessage;
            }

            try
            {
                $authHandler->checkCredentials($credentials->getUserId(), 
                    $credentials->getPassword(), 
                    $request );

                ThreadContext::setCallerCredentials($credentials);
            }
            catch(Exception $e)
            {
                $errorMessage = new ErrMessage( $this->messageId, $e );
                $errorMessage->faultCode = "Client.Authentication";
                
                return $errorMessage;
            }

            return new AckMessage($this->messageId, $this->clientId, null);          
        }
        else
        {

          if(is_null($this->body->getBody()))
          {
            $arr = array(0);
            $this->body->setBody($arr);
          }
          else if(!is_array($this->body->getBody()))
          {
            $arr = array($this->body->getBody());
            $this->body->setBody($arr);
          }

          try
          {
			$resolvedName = ServiceRegistry::getMapping( $this->destination );

			if( $resolvedName == "*" )
				$this->destination = $this->source;

            $returnValue = Invoker::handleInvoke(
                                                $request,
                                                $this->destination,
                                                $this->operation,
                                                $this->body->getBody());

            return new AckMessage( $this->messageId, $this->clientId, $returnValue );
          }
          catch( Exception $e )
          {
          	Log::log( LoggingConstants::EXCEPTION, "method invocation exception", $e );
          	return new ErrMessage( $this->messageId, $e );
          }
        }
      }
    }

?>