<?php
/*******************************************************************
 * AmfFormatter.php
 * Copyright (C) 2006 Midnight Coders, LLC
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * The software is licensed under the GNU General Public License (GPL)
 * For details, see http://www.gnu.org/licenses/gpl.txt.
 ********************************************************************/



require_once(WebOrb . "Writer/IProtocolFormatter.php");
require_once(WebOrb . "Writer/FlashorbBinaryWriter.php");
require_once(WebOrb . "Writer/ObjectSerializer.php");
require_once(WebOrb . "Message/Request.php");
require_once(WebOrb . "Util/ORBDateTime.php");
require_once(WebOrb . "Writer/ReferenceCache.php");

class AmfFormatter
    implements IProtocolFormatter
{

    private $m_stream;
    private $m_writer;
    private $m_objectSerializer;
    private $m_referenceCache;

    public function __construct()
    {
        $this->m_stream = "";
        $this->m_writer = new FlashorbBinaryWriter($this->m_stream);
        $this->m_objectSerializer = new ObjectSerializer();
        $this->m_referenceCache = new ReferenceCache();
    }

    public function beginWriteArray($length)
    {
        $this->m_writer->write(10);
        $this->m_writer->writeInt($length);
    }

    public function beginWriteFieldValue()
    {
    }

    public function beginWriteMessage(Request $message)
    {
    }

    public function beginWriteNamedObject($objectName, $fieldCount = null)
    {
        $this->m_writer->write(0x10);
        $this->directWriteString($objectName);
    }

    public function beginWriteObject($fieldCount = null)
    {
        $this->m_writer->write(3);
    }

    public function beginWriteObjectMap($size)
    {
        $this->m_writer->write(8);
        $this->m_writer->writeInt($size);
    }

    public function cleanup()
    {
    }

    public function directWriteBoolean($value)
    {
        $this->m_writer->write($value);
    }

    public function directWriteInt($value)
    {
        $this->m_writer->writeInt($value);
    }

    public function directWriteShort($value)
    {
        $this->m_writer->writeShort($value);
    }

    public function directWriteString($value)
    {
        $this->m_writer->writeUtf($value);
    }

    public function endWriteArray()
    {
    }

    public function endWriteFieldValue()
    {
    }

    public function endWriteMessage()
    {
    }

    public function endWriteNamedObject()
    {
        $this->m_writer->write(0);
        $this->m_writer->write(0);
        $this->m_writer->write(9);
    }

    public function endWriteObject()
    {
        $this->m_writer->write(0);
        $this->m_writer->write(0);
        $this->m_writer->write(9);
    }

    public function endWriteObjectMap()
    {
        $this->m_writer->write(0);
        $this->m_writer->write(0);
        $this->m_writer->write(9);
    }

    public function getBytes()
    {
        return $this->m_stream;
    }

    public function getContentType()
    {
        return "application/x-amf";
    }

    public function writeBoolean($value)
    {
        $this->m_writer->write(1);
        $this->m_writer->write($value);
    }

    public function writeDate(ORBDateTime $datetime)
    {
        $this->m_writer->write(11);
        $this->m_writer->writeDouble((float) ($datetime->getTotalMs()));
        $this->m_writer->writeShort((int) ($datetime->getTimeZone()));
    }

    public function writeFieldName($value)
    {
        $this->m_writer->writeUtf($value);
    }

    public function writeMessageVersion($value)
    {
        $this->m_writer->writeShort((int) $value);
    }

    public function writeNull()
    {
        $this->m_writer->write(5);
    }

    public function writeNumber($value)
    {
        $this->m_writer->write(0);

        $this->m_writer->writeDouble(floatval($value));
    }

    public function writeReference($refId)
    {
        $this->m_writer->write(7);
        $this->m_writer->writeShort($refId);
    }

    public function writeString($str)
    {
        $length = strlen($str);

        if($length < 65536)
        {
            $this->m_writer->write(2);
        }
        else
        {
            $this->m_writer->write(12);
        }

        $this->m_writer->writeUtf($str);
    }

    public function getObjectSerializer()
    {
        return $this->m_objectSerializer;
    }

    public function getReferenceCache()
    {
      return $this->m_referenceCache;
    }

    public function resetReferenceCache()
    {
      return $this->m_referenceCache->reset();
    }

    public function beginWriteBodyContent()
    {

    }

    public function endWriteBodyContent(){}

    public function writeArrayReference($refId)
    {
      $this->writeReference($refId);
    }

}

?>
