<?php
/*******************************************************************
 * FlashorbBinaryWriter.php
 * Copyright (C) 2006 Midnight Coders, LLC
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * The software is licensed under the GNU General Public License (GPL)
 * For details, see http://www.gnu.org/licenses/gpl.txt.
 ********************************************************************/



require_once(WebOrb . "Util/Logging/Log.php");

class FlashorbBinaryWriter
{

    private $m_stream;

    public function __construct(&$stream) 
    {
       $this->m_stream = &$stream;
    }

    public function write($value) 
    {
        $this->m_stream .= chr($value & 0xff);
    }

    public function writeDouble($value) 
    {
        $double = pack("d", $value);

        $this->m_stream .= strrev($double);
    }

    public function writeInt($value) 
    {
        $this->m_stream .= pack("N", $value);
    }

    public function writeShort($value) 
    {
        $this->m_stream .= pack("n", $value);
    }

    public function writeUtf($value, $long = FALSE)
    {
        $utfValue = $value;//utf8_encode($value);
        $strlen = strlen($utfValue);

        if ($strlen <= 65535)
        {
            $utfLen = $strlen;
        }
        else
        {
            $utfLen = $strlen << 1 | 0x1;
        }

        if (!$long && $utfLen > 65535)
        {
            throw new ApplicationException( "utf data format exception" );
        }

        if ($long)
        {
            $this->writeVarInt($strlen << 1 | 0x1);
        }
        else
        {
            $this->writeShort($strlen);
        }
        $this->m_stream .= $utfValue;
    }

    public function writeVarInt($v)
    {
        if ($v < 128)
        {
            $this->write($v);
        }
        else if ($v < 16384)
        {
            $this->write($v >> 7 & 0x7F | 0x80);
            $this->write($v & 0x7F);
        }
        else if ($v < 2097152)
        {
            $this->write($v >> 14 & 0x7F | 0x80);
            $this->write($v >> 7 & 0x7F | 0x80);
            $this->write($v & 0x7F);
        }
        else if ($v < 1073741824)
        {
            $this->write($v >> 22 & 0x7F | 0x80);
            $this->write($v >> 15 & 0x7F | 0x80);
            $this->write($v >> 8 & 0x7F | 0x80);
            $this->write($v & 0xFF);
        }
        else
        {
            throw new ServiceException("value out of range - " + $v);
        }
    }

    public function getStream() 
    {
        return $this->m_stream;
    }

}

?>
