<?php
/*******************************************************************
 * ObjectSerializer.php
 * Copyright (C) 2006 Midnight Coders, LLC
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * The software is licensed under the GNU General Public License (GPL)
 * For details, see http://www.gnu.org/licenses/gpl.txt.
 ********************************************************************/



require_once(WebOrb . "Writer/IObjectSerializer.php");
require_once(WebOrb . "Writer/IProtocolFormatter.php");
require_once(WebOrb . "Writer/MessageWriter.php");
require_once(WebOrb . "Util/Logging/Log.php");

class ObjectSerializer
    implements IObjectSerializer
{

    public function writeObject($className, $objectFields, IProtocolFormatter $writer)
    {
        if (is_null($className))
        {
            $writer->beginWriteObjectMap(count($objectFields));
        }
        else
        {
            $writer->beginWriteNamedObject($className, count($objectFields));
        }

        $keys = array_keys($objectFields);

        for ($i = 0; $i < count($objectFields); $i ++)
        {
            $fieldName = $keys[$i];

            $writer->writeFieldName($fieldName);
            $writer->beginWriteFieldValue();

            try
            {
                MessageWriter::writeObject($objectFields[$fieldName], $writer);
            }
            catch (Exception $exception)
            {
                Log::logException(LoggingConstants::ERROR, "unable to serialize object's field " . $fieldName, $exception);
            }

            $writer->endWriteFieldValue();
        }

        if (is_null($className))
        {
            $writer->endWriteObjectMap();
        }
        else
        {
            $writer->endWriteNamedObject();
        }
    }

}

?>
