<?php
/*******************************************************************
 * ObjectWriter.php
 * Copyright (C) 2006 Midnight Coders, LLC
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * The software is licensed under the GNU General Public License (GPL)
 * For details, see http://www.gnu.org/licenses/gpl.txt.
 ********************************************************************/



require_once(WebOrb . "Writer/ITypeWriter.php");
require_once(WebOrb . "Writer/IProtocolFormatter.php");
require_once(WebOrb . "Writer/MessageWriter.php");
require_once(WebOrb . "Util/Logging/Log.php");
require_once(WebOrb . "Writer/RemoteReferenceWriter.php");
require_once(WebOrb . "IRemote.php");

class ObjectWriter
    implements ITypeWriter
{

    private $m_remoteReferenceWriter;
    private $m_configured = false;
    private $m_serializePrivate = false;


    public function __construct()
    {
      $this->m_remoteReferenceWriter = new RemoteReferenceWriter();
    }

    public function isReferenceableType()
    {
        return true;
    }

    public function write(&$obj, IProtocolFormatter $writer)
    {

      if($obj instanceof IRemote)
      {
          $this->m_remoteReferenceWriter->write($obj,$writer);

          return;
      }

      if(!$this->m_configured)
      {
        $this->m_serializePrivate = false; //ThreadContext.getORBConfig().serializePrivateFields; - skipped
        $this->m_configured = true;
      }

      $className = get_class($obj);


      $objectClass = new ReflectionClass($className);

      $fullClassName = TypeLoader::getFullClassName($objectClass);

      $objectFields = array();
      $objectFields["_orbclassname"] = $fullClassName;

      $clientSideMapping = $this->getClientClass($className);

      if(!is_null($clientSideMapping))
      {
        $className = $clientSideMapping;
      }
      else
      {
        $className = $fullClassName;
      }

      Log::log(LoggingConstants::DEBUG, $className);

      // IAutoUpdate check - skipped

      while($objectClass)
      {
        $properties = $objectClass->getProperties();

         for ($i = 0; $i < count($properties); $i++)
         {
            if( !$properties[$i]->isPublic() )
              continue;

            if(!array_key_exists($properties[$i]->getName(),$objectFields))
            {
              $objectFields[$properties[$i]->getName()] = $properties[$i]->getValue($obj);
            }
         }

        $properties = $objectClass->getStaticProperties();

         for ($i = 0; $i < count($properties); $i++)
         {
            if(!array_key_exists($properties[$i]->getName(),$objectFields))
            {
              $objectFields[$properties[$i]->getName()] = $properties[$i]->getValue($obj);
            }
         }

         $objectClass = $objectClass->getParentClass();
      }

      $writer->getObjectSerializer()->writeObject($className, $objectFields, $writer);

    }


    private function getClientClass( $className )
    {
        return Types::getClientClassForServerType($className);
    }

}

?>
